/*
 * Kivio - Visual Modelling and Flowcharting
 * Copyright (C) 2000 theKompany.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#include "tkunits.h"

const char* unitNames[] = { "pt", "mm", "inch", "pica", "cm", "didot", "cicero" };
const char* unitLongNames[] = { "point", "millimeter", "inch", "pica", "cantimeter", "didot", "cicero" };

/*
 * Functions for converting Point <-> Inch <-> Millimeter
 * <-> Pica <-> Centimetre  <-> Didot <-> Cicero
 * 1 Inch = 72 pt = 6 pica = 25.4 mm = 67.54151050 dd =  5.628459208 cc
 */

float cvtPtToMm(float value) {
  return 25.4 * value / 72.0;
}

float cvtPtToCm(float value) {
  return 2.54 * value / 72.0;
}

float cvtPtToPica(float value) {
  return value / 12.0;
}

float cvtPicaToPt(float value) {
  return value * 12.0;
}

float cvtPtToInch(float value) {
  return value / 72.0;
}

float cvtPtToDidot(float value) {
  return value * 1238.0 / 1157.0; // 1157 dd = 1238 pt
}

float cvtPtToCicero(float value) {
  return value * 1238.0 / (1157.0 * 12.0); // 1 cc = 12 dd
}

float cvtInchToPt(float value) {
  return value * 72.0;
}

float cvtMmToPt(float value) {
  return value / 25.4 * 72.0;
}

float cvtCmToPt(float value) {
  return value / 2.54 * 72.0;
}

float cvtDidotToPt(float value) {
  return value * 1157.0 / 1238.0;
}

float cvtCiceroToPt(float value) {
  return value * (1157.0 * 12.0) / 1238.0;
}

float cvtPtToUnit(MeasurementUnit unit, float value) {
  switch (unit) {
  case UnitMillimeter:
    return cvtPtToMm (value);
    break;
  case UnitPica:
    return cvtPtToPica (value);
    break;
  case UnitInch:
    return cvtPtToInch (value);
    break;
  case UnitCentimeter:
    return cvtPtToCm (value);
    break;
  case UnitDidot:
    return cvtPtToDidot (value);
    break;
  case UnitCicero:
    return cvtPtToCicero (value);
    break;
  default:
    return value;
  }
}

float cvtUnitToPt(MeasurementUnit unit, float value) {
  switch (unit) {
  case UnitMillimeter:
    return cvtMmToPt (value);
    break;
  case UnitInch:
    return cvtInchToPt (value);
    break;
  case UnitCentimeter:
    return cvtCmToPt (value);
    break;
  case UnitDidot:
    return cvtDidotToPt (value);
    break;
  case UnitPica:
    return cvtPicaToPt (value);
    break;
  case UnitCicero:
    return cvtCiceroToPt (value);
    break;
  default:
    return value;
  }
}

const char* unitToString(MeasurementUnit unit)
{
  return unitNames[unit];
}

const char* unitToLongString(MeasurementUnit unit)
{
  return unitLongNames[unit];
}

void TKSize::set(double _w, double _h, MeasurementUnit _u )
{
  w = _w;
  h = _h;
  unit = _u;
}

double TKSize::widthToPt()
{
  return cvtUnitToPt(unit,w);
}

double TKSize::heightToPt()
{
  return cvtUnitToPt(unit,h);
}

void TKSize::convertToPt( int zoom )
{
  w = widthToPt()*zoom/100.0;
  h = heightToPt()*zoom/100.0;
  unit = UnitPoint;
}

double TKSize::widthToUnit( MeasurementUnit _u )
{
  return cvtPtToUnit(_u,widthToPt());
}

double TKSize::heightToUnit( MeasurementUnit _u )
{
  return cvtPtToUnit(_u,heightToPt());
}

void TKPoint::set(double _x, double _y, MeasurementUnit _u )
{
  x = _x;
  y = _y;
  unit = _u;
}

double TKPoint::xToPt()
{
  return cvtUnitToPt(unit,x);
}

double TKPoint::yToPt()
{
  return cvtUnitToPt(unit,y);
}

double TKPoint::xToUnit( MeasurementUnit _u )
{
  return cvtPtToUnit(_u,xToPt());
}

double TKPoint::yToUnit( MeasurementUnit _u )
{
  return cvtPtToUnit(_u,yToPt());
}
void TKPoint::convertToPt( int zoom )
{
  x = xToPt()*zoom/100.0;
  y = yToPt()*zoom/100.0;
  unit = UnitPoint;
}
