/*
 * Kivio - Visual Modelling and Flowcharting
 * Copyright (C) 2000 theKompany.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#include "tool_text.h"

#include <qcursor.h>
#include <kiconloader.h>
#include <kstddirs.h>

#include "kivio_view.h"
#include "kivio_canvas.h"
#include "kivio_page.h"
#include "kivio_doc.h"
#include "stencil_text_dlg.h"

#include "kivio_stencil_spawner_set.h"
#include "kivio_stencil_spawner.h"
#include "kivio_custom_drag_data.h"
#include "kivio_layer.h"
#include "kivio_point.h"
#include "kivio_stencil.h"

TextTool::TextTool( KivioView* view )
:Tool(view)
{
  setSortNum(2);

  ToolSelectAction* text = new ToolSelectAction( actionCollection(), "ToolAction" );
  KAction* m_z1 = new KAction( i18n("Edit Stencil Text"), "kivio_text", CTRL+Key_T, actionCollection(), "text" );
  text->insert(m_z1);

  m_mode = stmNone;

  QPixmap pix = BarIcon("kivio_text_cursor");
  m_pTextCursor = new QCursor(pix,2,2);
}

TextTool::~TextTool()
{
}


/**
 * Event delegation
 *
 * @param e The event to be identified and processed
 *
 */
void TextTool::processEvent( QEvent* e )
{
    switch (e->type())
    {
    case QEvent::MouseButtonPress:
        mousePress( (QMouseEvent*)e );
        break;

    case QEvent::MouseButtonRelease:
        mouseRelease( (QMouseEvent*)e );
        break;

    case QEvent::MouseMove:
        mouseMove( (QMouseEvent*)e );
        break;

    default:
      break;
    }
}

void TextTool::activate()
{
    debug("TextTool activate");
    m_pCanvas->setCursor(*m_pTextCursor);
    m_mode = stmNone;
    
    KivioPage *pPage =m_pView->activePage();
    KivioStencil *pStencil = pPage->selectedStencils()->first();

    if( !pStencil )
        return;
    
    setStencilText();
    controller()->activateDefault();
}

void TextTool::setStencilText()
{
    KivioDoc* m_pDoc = m_pView->doc();
    KivioPage *pPage =m_pView->activePage();
    KivioStencil *pStencil = pPage->selectedStencils()->first();

    if( !pStencil )
        return;

    KivioStencilTextDlg d(0, pStencil->text());

    if( !d.exec() )
        return;

    QString text = d.text();

    while( pStencil )
    {
        pStencil->setText( text );
        pStencil = pPage->selectedStencils()->next();
    }

    m_pDoc->setModified(true);
    m_pCanvas->repaint();
}

void TextTool::deactivate()
{
}

void TextTool::configure()
{
}


void TextTool::text(QRect r)
{
    // Calculate the start and end clicks in terms of page coordinates
    TKPoint startPoint = m_pCanvas->mapFromScreen( QPoint( r.x(), r.y() ) );
    TKPoint releasePoint = m_pCanvas->mapFromScreen( QPoint( r.x() + r.width(), r.y() + r.height() ) );


    float x, y, w, h;

    // Calculate the x,y position of the textion box
    x = startPoint.x < releasePoint.x ? startPoint.x : releasePoint.x;
    y = startPoint.y < releasePoint.y ? startPoint.y : releasePoint.y;

    // Calculate the w/h of the textion box
    w = releasePoint.x - startPoint.x;
    if( w < 0.0f )
        w *= -1.0f;

    h = releasePoint.y - startPoint.y;
    if( h < 0.0f )
        h *= -1.0f;

    KivioDoc* doc = m_pView->doc();
    KivioPage* pPage = m_pCanvas->activePage();
    
    KivioStencilSpawner* ss = doc->findInternalStencilSpawner("Text");
    if (!ss)
        return;
        
    KivioStencil* pStencil = ss->newStencil();
    pStencil->setPosition(x,y);
    pStencil->setDimensions(w,h);
    pStencil->setText("");
    pPage->unselectAllStencils();
    pPage->addStencil(pStencil);
    pPage->selectStencil(pStencil);
    
    // Mark the page as modified and repaint
    m_pCanvas->repaint();
    
    setStencilText();
    
    if (pStencil->text().isEmpty()) {
        pPage->deleteSelectedStencils();    
    }
    else
    {
      // Only set modified to true if we didn't cancel
      doc->setModified(true);
    }
        
    m_pCanvas->repaint();
}

void TextTool::mousePress( QMouseEvent *e )
{
    if( startRubberBanding( e ) )
    {
        m_mode = stmDrawRubber;
    }
}


/**
 * Tests if we should start rubber banding (always returns true).
 */
bool TextTool::startRubberBanding( QMouseEvent *e )
{
    m_pCanvas->startRectDraw( e->pos(), KivioCanvas::Rubber );
    m_pCanvas->repaint();

    return true;
}

void TextTool::mouseMove( QMouseEvent * e )
{
    switch( m_mode )
    {
        case stmDrawRubber:
            continueRubberBanding(e);
            break;
        
        default:
            break;
    }
}

void TextTool::continueRubberBanding( QMouseEvent *e )
{
    m_pCanvas->continueRectDraw( e->pos(), KivioCanvas::Rubber );
}

void TextTool::mouseRelease( QMouseEvent *e )
{
    m_releasePoint = e->pos();

    switch( m_mode )
    {
        case stmDrawRubber:
            endRubberBanding(e);
            break;
    }
	
  	m_mode = stmNone;

    m_pCanvas->repaint();
}

void TextTool::endRubberBanding(QMouseEvent *e)
{
    // End the rubber-band drawing
    m_pCanvas->endRectDraw();

    // We can't text if the start and end points are the same
    if( m_startPoint != m_releasePoint )
    {
        text(m_pCanvas->rect());
    }
}
